/* =================================================== */
/*                usb_init                             */
/* =================================================== */
void usb_init()
{
 /* configure_usb_clock */
 Pll_set_div(PLL_24MHz);
 Pll_enable();

 /* 祭 USB */
 Usb_enable();

 /* ⪫-  砫 㬥樨 */
 Usb_detach();
 delay5();
 Usb_attach();
 delay5();

 usb_configure_endpoint(0, CONTROL|MSK_EPEN);
 usb_reset_endpoint(0);
 usb_configuration_nb = 0x00;
 Usb_enable_ep_int(0);
}

/* =================================================== */
/*                usb_enumeration_process              */
/* =================================================== */
void usb_enumeration_process()
{ 
  Usb_select_ep(0);
  usb_read_request();
}

/* =================================================== */
/*                    usb_read_request                 */
/* =================================================== */
void usb_read_request()
{ 
  data uint16 wRequest;

  /* ⥭ bmRequestType */
  ((byte*)&wRequest)[0] = Usb_read_byte();   
  /* ⥭ bRequest      */
  ((byte*)&wRequest)[1] = Usb_read_byte();

  switch (wRequest)  
  {
    case GET_STATUS_DEVICE:
     usb_get_status_device();   
     break;
    case GET_STATUS_INTERF:     
     usb_get_status_interface();   
     break;
    case GET_STATUS_ENDPNT:     
     usb_get_status_endpoint();   
     break;

    case CLEAR_FEATURE_DEVICE:
     STALL();
     break;
    case CLEAR_FEATURE_INTERF:  
     STALL();
     break;
    case CLEAR_FEATURE_ENDPNT:  
     STALL();
     break;
    case SET_FEATURE_DEVICE:
     STALL();
     break;
    case SET_FEATURE_INTERF:    
     STALL();
     break;
    case SET_FEATURE_ENDPNT:    
     STALL();
     break;

    case SET_ADDRESS:
     usb_set_address();   
     break;

    case GET_DESCRIPTOR_DEVICE:
    case GET_DESCRIPTOR_INTERF:
    case GET_DESCRIPTOR_ENDPNT:
     usb_get_descriptor(); 
     break;

    case SET_DESCRIPTOR:        
     STALL();
     break;

    case GET_CONFIGURATION:
     usb_get_configuration();         
     break;
    case SET_CONFIGURATION:     
     usb_set_configuration();
     break;

    case GET_INTERFACE:
     STALL();
     break;
    case SET_INTERFACE:         
     STALL();
     break;
    case SYNCH_FRAME:
     STALL();
     break;

    case GET_REPORT:
     hid_get_report();
     break;

    default:
     STALL();
     break;
  }
}

/* =================================================== */
/*                usb_get_status_xxx                   */
/* =================================================== */
void usb_get_status_device(){
  Usb_clear_rx_setup();
  Usb_set_DIR();

  /*
     [1]= 0: ன⢮  ᨣ 㤪
     [0]= 0: ன⢮ 砥 ⠭  設 USB
  */
  Usb_write_byte(0x00);  
  /* १ࢨ஢, ᥣ 0 */
  Usb_write_byte(0x00);

  END_OK();
}

void usb_get_status_interface(){
  Usb_clear_rx_setup();
  Usb_set_DIR();

  /* १ࢨ஢, ᥣ 0 */
  Usb_write_byte(0x00);          
  Usb_write_byte(0x00);

  END_OK();
}

void usb_get_status_endpoint(){
  data byte wIndex;

  /*  ᯮ */
  ACC = Usb_read_byte();  
  ACC = Usb_read_byte();  

  /*  筮 窨 */
  wIndex = Usb_read_byte();
  /* 浪  筮 窨 */
  wIndex = wIndex & MSK_EP_DIR;

  Usb_clear_rx_setup();
  Usb_set_DIR();

  /* 
    [0] ﭨ 筮 窨:
         0 - 筠 窠 㭪樮 ଠ쭮
         1 - ।  ஢
  */
  Usb_write_byte(0x00);
  /* १ࢨ஢, ᥣ 0 */
  Usb_write_byte(0x00);

  END_OK();
}

/* =================================================== */
/*                    usb_get_descriptor               */
/* =================================================== */
void usb_get_descriptor(){

 data byte    data_to_transfer;
 data uint16  wLength;
 data byte   descriptor_type;
 data byte   string_type;
 bit          zlp;
 data byte   *pbuffer;

 zlp = FALSE;    

 /* ⨯ STRING-ਯ */
 string_type     = Usb_read_byte();  
 /* ⨯ ਯ */
 descriptor_type = Usb_read_byte();  

 switch (descriptor_type)
 {
    /* ਯ ன⢠ */
    case DEVICE: 
    {
      data_to_transfer = sizeof(usb_device_descriptor);
      pbuffer = &(usb_device_descriptor.bLength);
      break;
    }
    /* ਯ 䨣樨 */
    case CONFIGURATION: 
    {
      data_to_transfer = sizeof(usb_configuration);
      pbuffer = &(usb_configuration.cfg.bLength);
      break;
    }
    /* HID-ਯ */
    case HID:
    {
      data_to_transfer = sizeof(usb_configuration.hid);
      pbuffer = &(usb_configuration.hid.bLength);
      break;
    }
    /* ਯ ९ */
    case REPORT:
    {
      data_to_transfer = SIZE_OF_REPORT;
      pbuffer = &(HIDReport.rep[0]);
      break;
    }
    /* ਯ ப */
    case STRING:
    {
      switch (string_type)
      {
        case LANG_ID:
        {
          data_to_transfer = sizeof (usb_language);
          pbuffer = &(usb_language.bLength);
          break;
        }
        case MAN_INDEX:
        {
          data_to_transfer = sizeof (usb_manufacturer);
          pbuffer = &(usb_manufacturer.bLength);
          break;
        }
        case PRD_INDEX:
        {
          data_to_transfer = sizeof (usb_product);
          pbuffer = &(usb_product.bLength);
          break;
        }
        case SRN_INDEX:
        {
          data_to_transfer = sizeof (usb_serial_number);
          pbuffer = &(usb_serial_number.bLength);
          break;
        }
        default:
        {
          STALL();
          return;
        }
      }
      break;
    }

    default:
    { 
      STALL();
      return;
    }
  }

  /*  wIndex,  ᯮ */
  ACC = Usb_read_byte();  
  ACC = Usb_read_byte();

  /* ⥭ ॡ㥬 ᫠  */
  /* ( wLenght)               */
  ((byte*)&wLength)[1] = Usb_read_byte();   
  ((byte*)&wLength)[0] = Usb_read_byte();

  /* ॡ  祬 ? */
  if (wLength > data_to_transfer)
  {
    /* ᫨ ᫮   ⭮ ࠧ , */
    /*   襭 ନ㥬 ᯥ樠쭮      */
    if ((data_to_transfer % EP_CONTROL_LENGTH) == 0) { zlp = TRUE; }
    else { zlp = FALSE; }                        
  }
  else
  {
    /* 뫠 ⮫쪮 ॡ㥬 ᫮  */
    /*   , 祬  ॠ쭮  */
    /*     ॡ        */
    data_to_transfer = (byte)wLength;      
  }

  /*  䫠 ਥ  */
  Usb_clear_rx_setup() ;       
  /* ४祭 ࠢ 㫥 窨 */
  Usb_set_DIR();                            

  /* 諥  墠⠥   ନ஢   */
  /* ⮪  뫠 ⮬ (᫨ )               */
  while (data_to_transfer > EP_CONTROL_LENGTH)
  {
    /* ।  ᨬ쭮 */
    pbuffer = usb_send_ep0_packet(pbuffer, EP_CONTROL_LENGTH);
    /* ᤢ 㪠⥫ */
    data_to_transfer -= EP_CONTROL_LENGTH;

    /*  襭 । */
    while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));
    Usb_clear_tx_complete();
    if ((Usb_rx_complete())) /* ᫨ । ࢠ ⮬ */
    {
      Usb_clear_tx_ready(); /*  । */
      Usb_clear_rx();
      return;
    }
  }

  /* 뫠             */
  /* ᫨ ⠪  ,    */
  /* ⮬ 襭 ।               */ 
  pbuffer = usb_send_ep0_packet(pbuffer, data_to_transfer);
  data_to_transfer = 0;
  while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));
  Usb_clear_tx_complete();
  if ((Usb_rx_complete())) /* ᫨ । ࢠ ⮬ */
  {
    Usb_clear_tx_ready();
    Usb_clear_rx();
    return;
  }

  /*  室 (᫨   )       */
  /* ନ㥬  襭 ᯥ樠쭮 - 뫪 */
  /*  㫥                              */
  if (zlp == TRUE)
  {
    usb_send_ep0_packet(pbuffer, 0);
    while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));
    Usb_clear_tx_complete();
    if ((Usb_rx_complete())) /* ᫨ । ࢠ ⮬ */
    {
      Usb_clear_tx_ready();
      Usb_clear_rx();
      Usb_clear_DIR();  
      return;
    }

  }
  /*  襭 । */
  while ((!(Usb_rx_complete())) && (!(Usb_setup_received())));

  if (Usb_setup_received())
  {
    return;
  }

  if (Usb_rx_complete())
  {
    Usb_clear_DIR();  /* ४祭 ࠢ 0 窨 */
    Usb_clear_rx();
  }

}

/* =================================================== */
/*                  usb_get_configuration              */
/* =================================================== */
void usb_get_configuration(){
  Usb_clear_rx_setup();
  Usb_set_DIR();
  Usb_write_byte(usb_configuration_nb);
  END_OK();
}

/* =================================================== */
/*                  usb_set_configuration              */
/* =================================================== */
void usb_set_configuration(){
  data byte configuration_number;

  /*   䨣樨 */
  configuration_number = Usb_read_byte();   
  Usb_clear_DIR();
  Usb_clear_rx_setup();

  /* ᫨ ࠭ 㯭 䨣 */
  if (configuration_number <= CONF_NB)
  {
    /* ࠭  䨣樨 */
    usb_configuration_nb = configuration_number;
  }
  else
  {
    /* 訡  - ⪫塞 */
    STALL();
    return;
  }

  Usb_set_tx_ready();            
  while (!Usb_tx_complete());
  Usb_clear_tx_complete();

  /* 䨣஢ 㣨  祪 */
  usb_ep_init();                 
}

/* =================================================== */
/*                  usb_set_address                    */
/* =================================================== */
void usb_set_address(){
  data byte add;

  /* ࠭  */
  add = Usb_read_byte();    
  Usb_clear_rx_setup();
  Usb_set_tx_ready();      
 
  /* ⠢ 䫠 "ன⢮ ᮢ" */
  Usb_set_FADDEN();        
  while (!(Usb_tx_complete()));
  Usb_clear_tx_complete();
  Usb_clear_DIR();

  /* ⠭ 祭  */
  Usb_configure_address(add);
}

/* =================================================== */
/* =================================================== */
void hid_get_report()
{
 data byte bReportType;
 data byte bReportID;

 if (usb_configuration_nb == 0){
  STALL();
  return;
 }

 P3.3=p_test; p_test=!p_test;

 bReportID   = Usb_read_byte();
 bReportType = Usb_read_byte();

 Usb_clear_rx_setup();
 Usb_set_DIR();

 switch (bReportID)
 {
  case 1:
  {
    Usb_write_byte(0x01);
    Usb_write_byte(0x02);
    Usb_write_byte(point1_state);
    Usb_write_byte(0x04);
    Usb_write_byte(0x05);
    Usb_write_byte(0x06);
    Usb_write_byte(0x07);
    break;
  }

  case 2:
  {
    Usb_write_byte(0x11);
    Usb_write_byte(0x12);
    Usb_write_byte(point1_state);
    Usb_write_byte(0x14);
    break;
  }
  default:
    STALL();
    return;
 }
 point1_state++;
 END_OK();
}
/* =================================================== */
/* 䨣஢  祪, ஬ ᭮     */
/* =================================================== */
void usb_ep_init()
{
  usb_configure_endpoint(1, EP_1_CONFIG | MSK_EPEN);
  usb_reset_endpoint(1);
  Usb_enable_ep_int(1);
}
