// ATDriverDevice.cpp
// Implementation of ATDriverDevice device class
//
// Generated by DriverWizard version DriverStudio 2.6.0 (Build 336)
// Requires Compuware's DriverWorks classes
//

#pragma warning(disable:4065) // Allow switch statement with no cases
		  
#include <vdw.h>
#include <kusb.h>

#include "ATDriver.h"
#include "ATDriverDevice.h"
#include "..\ATDriverioctl.h"

#pragma hdrstop("ATDriver.pch")


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::ATDriverDevice
////////////////////////////////////////////////////////////////////////
ATDriverDevice::ATDriverDevice(PDEVICE_OBJECT Pdo, ULONG Unit) :
	KPnpDevice(Pdo, NULL)
{
    DbgPrint("ATDriverDevice::ATDriverDevice START");
    NTSTATUS status;

	// Check constructor status
    if ( ! NT_SUCCESS(m_ConstructorStatus) )
    {
    	return;
    }

	DbgPrint("Unit number is %d", Unit);
	// Remember our unit number
	m_Unit = Unit;

	// Initialize the lower device
	status = m_Lower.Initialize(this, Pdo);

    if (status == STATUS_SUCCESS){
    	DbgPrint("m_Lower.Initialize OK");
    } else {
    	DbgPrint("ERROR: m_Lower.Initialize");
    }

	status = m_Interface.Initialize(
			m_Lower,	// KUsbLowerDevice
			0,			// InterfaceNumber
			1,			// ConfigurationValue 
			0			// Initial Interface Alternate Setting
	); 

   	if (status == STATUS_SUCCESS){
   		DbgPrint("m_Interface.Initialize OK");
   	} else {
   		DbgPrint("ERROR: m_Interface.Initialize");
   	}

	// Initialize each Pipe object
	status = m_Endpoint1IN.Initialize(m_Lower, 0x81 /* ,8 -  ! */); 

   	if (status == STATUS_SUCCESS){
   		DbgPrint("m_Endpoint1IN.Initialize OK");
   	} else {
   		DbgPrint("ERROR: m_Endpoint1IN.Initialize");
   	}

        // Inform the base class of the lower edge device object
	SetLowerDevice(&m_Lower);
	// Initialize the PnP Policy settings to the "standard" policy
	SetPnpPolicy();

   	DbgPrint("ATDriverDevice::ATDriverDevice END");
}


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::~ATDriverDevice
////////////////////////////////////////////////////////////////////////
ATDriverDevice::~ATDriverDevice()
{
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::DefaultPnp
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::DefaultPnp(KIrp I) 
{
	I.ForceReuseOfCurrentStackLocationInCalldown();
	return m_Lower.PnpCall(this, I);
}


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::DefaultPower
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::DefaultPower(KIrp I) 
{
	I.IndicatePowerIrpProcessed();
	I.CopyParametersDown();
	return m_Lower.PnpPowerCall(this, I);
}

////////////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::SystemControl
////////////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::SystemControl(KIrp I) 
{
	I.ForceReuseOfCurrentStackLocationInCalldown();
	return m_Lower.PnpCall(this, I);
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::OnStartDevice
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::OnStartDevice(KIrp I)
{
   	DbgPrint("ATDriverDevice::OnStartDevice START");

	NTSTATUS status = STATUS_UNSUCCESSFUL;

	AC_STATUS acStatus = AC_SUCCESS;

	I.Information() = 0;

   	DbgPrint("Activate configuration 1...");
	acStatus = m_Lower.ActivateConfiguration(
		1	// ConfigurationValue 1 (the first configuration)
	);

	if ((acStatus == AC_SUCCESS) || (acStatus == AC_FAILED_TO_OPEN_PIPE_OBJECT))
	{
		if (acStatus == AC_SUCCESS)
	    		DbgPrint("Activate configuration OK");
		if (acStatus == AC_FAILED_TO_OPEN_PIPE_OBJECT)
	    		DbgPrint("ERROR: AC_FAILED_TO_OPEN_PIPE_OBJECT");
		status = STATUS_SUCCESS;
	}

    DbgPrint("ATDriverDevice::OnStartDevice END");
   	return status;  // base class completes the IRP
}


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::OnStopDevice
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::OnStopDevice(KIrp I)
{
    DbgPrint("ATDriverDevice::OnStopDevice START");
	NTSTATUS status = STATUS_SUCCESS;

	m_Lower.DeActivateConfiguration();

    DbgPrint("ATDriverDevice::OnStopDevice END");
	return status;
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::OnRemoveDevice
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::OnRemoveDevice(KIrp I)
{
    DbgPrint("ATDriverDevice::OnRemoveDevice START");

	// Device removed, release the system resources 
	// used by the USB lower device.
	m_Lower.ReleaseResources();

    DbgPrint("ATDriverDevice::OnRemoveDevice END");
	return STATUS_SUCCESS;
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::Create
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::Create(KIrp I)
{
	NTSTATUS status;
   	DbgPrint("ATDriverDevice::Create START");

// TODO: Add driver specific create handling code here

	// Generally a create IRP is targeted at our FDO, so we don't need
	// to pass it down to the PDO.  We have found for some devices, the
	// PDO is not expecting this Irp and returns an error code.
	// The default wizard code, therefore completes the Irp here using
	// PnpComplete().  The following commented code could be used instead
	// of PnpComplete() to pass the Irp to the PDO, which would complete it.
	//
//	I.ForceReuseOfCurrentStackLocationInCalldown();
//	status = m_Lower.PnpCall(this, I);

	status = I.PnpComplete(this, STATUS_SUCCESS, IO_NO_INCREMENT);

   	DbgPrint("ATDriverDevice::Create END");
	return status;
}


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::Close
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::Close(KIrp I)
{
	NTSTATUS status;
   	DbgPrint("ATDriverDevice::Close START");

// TODO: Add driver specific close handling code here

	// Generally a close IRP is targeted at our FDO, so we don't need
	// to pass it down to the PDO.  We have found for some devices, the
	// PDO is not expecting this Irp and returns an error code.
	// The default wizard code, therefore completes the Irp here using
	// PnpComplete().  The following commented code could be used instead
	// of PnpComplete() to pass the Irp to the PDO, which would complete it.
	//
//	I.ForceReuseOfCurrentStackLocationInCalldown();
//	status = m_Lower.PnpCall(this, I);

	status = I.PnpComplete(this, STATUS_SUCCESS, IO_NO_INCREMENT);

    DbgPrint("ATDriverDevice::Close END");
    return status;
}


////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::Read
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::Read(KIrp I) 
{
   	DbgPrint("ATDriverDevice::Read START");
	// Always ok to read 0 elements.
	if (I.ReadSize() == 0)
	{
		I.Information() = 0;
   		DbgPrint("WARNING: I.Information() = 0");
   		DbgPrint("ATDriverDevice::Read END");
		return I.PnpComplete(this, STATUS_SUCCESS);
	}
	NTSTATUS status		= STATUS_SUCCESS;

	PVOID	pBuffer		= I.BufferedReadDest();

	ULONG   dwTotalSize = I.ReadSize(CURRENT);	// Requested read size
	ULONG   dwBytesRead = 0;			// Count of bytes read

// TODO:	If the read can be satisfied immediately, set the Information
//			and Status fields now, then call NextIrp to complete this IRP
//			and start processing the next IRP in the queue.

// TODO:	If the data is not yet available, initiate a request to the
//			physical device here, and defer the Information, Status,
//			and NextIrp handling until the hardware indicates that the
//			read is complete.  Typically, this might be handled in a
//			DPC that is called after the hardware finishes transferring
//			the data.

// TODO:	To satisfy the read now, transfer data from the device to
//			caller's buffer at "pBuffer".  Then, indicate how much data was
//			transferred:

	ULONG dwMaxSize = m_Endpoint1IN.MaximumTransferSize();
	DbgPrint(" READ: Requested=%d MaxSize=%d", dwTotalSize, dwMaxSize);

    if (dwTotalSize > dwMaxSize)
    {
		dwTotalSize = dwMaxSize;
    }


	for (int i=0; i<8; i++)
		((PCHAR)(pBuffer))[i] = i;

/*
        // Create an URB to do actual read from the pipe
        PURB pUrb = m_Endpoint1IN.BuildControlTransfer(
                       	pBuffer,        // Where is data coming from?
                        dwTotalSize,    // How much data to read?
                        TRUE            // TRUE = IN
                    );
*/
/*
        PURB pUrb = m_Endpoint1IN.BuildBulkTransfer(
                       	pBuffer,        // Where is data coming from?
                        dwTotalSize,    // How much data to read?
                        TRUE,           // TRUE = IN
                        NULL,
                        FALSE,          // Short
                        NULL
                    );
*/


    PURB pUrb = m_Endpoint1IN.BuildInterruptTransfer(
                       	pBuffer,        // Where is data coming from?
                        dwTotalSize,    // How much data to read?
                        TRUE            // Short OK
				);

    if (pUrb == NULL)
    {
		I.Information() = 0;
    	DbgPrint("ERROR: BuildInterrupt");
        return I.PnpComplete(this, STATUS_INSUFFICIENT_RESOURCES);
	}

    // Submit the URB to our USB device, synchronously - say less is OK
  	pUrb->UrbBulkOrInterruptTransfer.TransferFlags =
             	(USBD_TRANSFER_DIRECTION_IN | USBD_SHORT_TRANSFER_OK);
/*
     	pUrb->UrbControlTransfer.TransferFlags = USBD_TRANSFER_DIRECTION_IN;
*/

	if (!m_Endpoint1IN.IsOpen())
	{
		I.Information() = 0;
    	DbgPrint("ERROR: m_Endpoint1IN is not open!");
        return I.PnpComplete(this, STATUS_INSUFFICIENT_RESOURCES);
	}

    status = m_Endpoint1IN.SubmitUrb(pUrb);


	dwBytesRead = pUrb->UrbBulkOrInterruptTransfer.TransferBufferLength;
/*
	dwBytesRead = pUrb->UrbControlTransfer.TransferBufferLength;
*/
	if ( status == STATUS_SUCCESS) {
		DbgPrint("Read SUCCESS %d bytes", dwBytesRead);
	} 
	else {
		DbgPrint("ERROR: SubmitUrb (%d) error code=%X", dwBytesRead, status);
	}

	for (i=0; i<8; i++)
		DbgPrint("  %d", ((PCHAR)(pBuffer))[i] );

	delete pUrb;

	I.Information() = dwBytesRead;

	DbgPrint("ATDriverDevice::Read END");
	return I.PnpComplete(this, status);
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::DeviceControl
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::DeviceControl(KIrp I) 
{
	NTSTATUS status;
	switch (I.IoctlCode())
	{
		case ATDRIVER_IOCTL_800:
			status = ATDRIVER_IOCTL_800_Handler(I);
			break;

		default:
			// Unrecognized IOCTL request
			status = STATUS_INVALID_PARAMETER;
			break;
	}

	// If the IRP's IOCTL handler deferred processing using some driver
	// specific scheme, the status variable is set to STATUS_PENDING.
	// In this case we simply return that status, and the IRP will be
	// completed later.  Otherwise, complete the IRP using the status
	// returned by the IOCTL handler.
	if (status == STATUS_PENDING)
	{
		return status;
	}
	else
	{
		return I.PnpComplete(this, status);
	}
}

////////////////////////////////////////////////////////////////////////
//  ATDriverDevice::ATDRIVER_IOCTL_800_Handler
////////////////////////////////////////////////////////////////////////
NTSTATUS ATDriverDevice::ATDRIVER_IOCTL_800_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

// TODO:	Verify that the input parameters are correct
//			If not, return STATUS_INVALID_PARAMETER

// TODO:	Handle the the ATDRIVER_IOCTL_800 request, or 
//			defer the processing of the IRP (i.e. by queuing) and set
//			status to STATUS_PENDING.

// TODO:	Assuming that the request was handled here. Set I.Information
//			to indicate how much data to copy back to the user.
	I.Information() = 0;

	return status;
}



