#include "descript.h"

#define USB_SPECIFICATION     0x0100
#define DEVICE_CLASS          0x00
#define DEVICE_SUB_CLASS      0x00
#define DEVICE_PROTOCOL       0x00
#define EP_CONTROL_LENGTH     8
#define VENDOR_ID             0xEB03   /* Atmel vendor ID = 03EBh */
#define PRODUCT_ID            0x0358
#define RELEASE_NUMBER        0x0000

#define LANG_ID              0x00
#define MAN_INDEX            0x01 
#define PRD_INDEX            0x02 
#define SRN_INDEX            0x03 


code struct usb_st_device_descriptor usb_device_descriptor =
{ 
  sizeof(usb_device_descriptor), 
  DEVICE,             /* =1 */
  USB_SPECIFICATION,  /* ᯥ䨪     */
  DEVICE_CLASS,       /*  ன⢠ */
  DEVICE_SUB_CLASS, 
  DEVICE_PROTOCOL,    /* ⮪ USB     */
  EP_CONTROL_LENGTH,  /* ࠧ  0- 窨 */
  VENDOR_ID,          /* ID ந⥫  ன⢠ */
  PRODUCT_ID,
  RELEASE_NUMBER,     /*  ன⢠ */
  MAN_INDEX,          /* . ப ⮢⥫  */
  PRD_INDEX,          /* . ப த      */
  SRN_INDEX,          /* . ப ਩  */
  1                   /* ᫮ 䨣権 */
};

#define CONF_LENGTH           wSWAP(25) 
#define CONF_NB               1
#define CONF_ATTRIBUTES       USB_CONFIG_BUSPOWERED
#define MAX_POWER             50  /* = 100 mA */

/* INTERFACE 0 DESCRIPTOR */
#define INTERFACE0_CLASS       0x00 
#define INTERFACE0_SUB_CLASS   0xFF
#define INTERFACE0_PROTOCOL    0xFF

/* ࢠ 筠 窠 */
#define EP_1_CONFIG       (INTERRUPT|EP_CONFIG_IN)  /* 䨣 */
#define EP_1_ADDRESS      (1|EP_DIRECT_IN)          /*         */
#define EP_1_ATTRIBUTES   INTERRUPT                 /* ਡ    */

code struct  
{
  struct usb_st_configuration_descriptor  cfg;
  struct usb_st_interface_descriptor      ifc;
  struct usb_st_endpoint_descriptor       ep1;
 }
 usb_configuration =
 {
/* CONFIGURATION */
    { 9, 
      CONFIGURATION,   /* =2 */
      CONF_LENGTH,     /*   ਯ஢ */
      1,               /* ᫮ 䥩ᮢ       */
      CONF_NB,         /*  䨣樨      */
      0,               /* . ப 䨣  */
      CONF_ATTRIBUTES, /* ਡ               */
      MAX_POWER        /* 100 mA                  */
    },
/* INTERFACE 0 */
    { 9, 
      INTERFACE,       /* =4 */
      0,               /* 0- 䥩              */
      0,               /*  ୠ⨢ . */
      1,               /* 1 筠 窠 (஬ 0) */
      INTERFACE0_CLASS,
      INTERFACE0_SUB_CLASS, 
      INTERFACE0_PROTOCOL, 
      0                /* . ப 䥩   */
    },
/* ਯ ࢮ 筮 窨 */
    {
      7, 
      ENDPOINT,          /* ਯ ENDPOINT        */
      EP_1_ADDRESS,      /*  筮 窨       */
      EP_1_ATTRIBUTES,   /* ਡ 筮 窨   */
      wSWAP(8),          /* ᨬ ࠧ  */
      1                  /*               */
    }
   };


/* ਯ ப */
#define structSTRING_DESCRIPTOR(NAME, LEN) struct NAME \
                                           { \
                                             byte   bLength; \
                                             byte   bDescriptorType; \
                                             uint16 wstring[LEN]; \
                                            }


/* ਯ ப ந⥫ */
#define USB_MANUFACTURER_NAME {'P'<<8, 'V'<<8, 'A'<<8, \
                               'S'<<8, 'o'<<8, 'f'<<8, 't'<<8}
#define USB_MN_LENGTH         7
code structSTRING_DESCRIPTOR(usb_st_manufacturer, USB_MN_LENGTH)
 usb_manufacturer =
  { sizeof(usb_manufacturer),  STRING, USB_MANUFACTURER_NAME };

/* ਯ ப த */

#define USB_PRODUCT_NAME      {'P'<<8, 'V'<<8, 'A'<<8, 'S'<<8, 'o'<<8, 'f'<<8, 't'<<8, \
                               ' '<<8,  \
                               't'<<8, 'e'<<8, 's'<<8, 't'<<8, \
                               ' '<<8, \
                               'b'<<8, 'o'<<8, 'a'<<8, 'r'<<8, 'd'<<8}
#define USB_PN_LENGTH         18

/* ᪮  "⮢ "
#define USB_PRODUCT_NAME {0x2204, 0x3504, 0x4104, 0x4204, \
                          0x3E04, 0x3204, 0x3004, 0x4F04, 0x2000, \
                          0x3F04, 0x3B04, 0x3004, 0x4204, 0x3004}
#define USB_PN_LENGTH         14
*/

code structSTRING_DESCRIPTOR(usb_st_product, USB_PN_LENGTH)
 usb_product =
  { sizeof(usb_product), STRING, USB_PRODUCT_NAME };


/* ਯ ப ਩  */
#define USB_SERIAL_NUMBER     {'1'<<8, '.'<<8, '0'<<8, '.'<<8, '0'<<8}
#define USB_SN_LENGTH         5
code structSTRING_DESCRIPTOR(usb_st_serial_number, USB_SN_LENGTH)
 usb_serial_number =
  { sizeof(usb_serial_number), STRING, USB_SERIAL_NUMBER };

/* ਯ ப 䨪 몠 */
#define LANGUAGE_ID           0x0904

code structSTRING_DESCRIPTOR(usb_st_language_descriptor, 1)
 usb_language =
 { sizeof(usb_language), STRING, LANGUAGE_ID };
