Unit ContList;

INTERFACE

Uses Crt, PCI;

{ USB ஫ }
Type
 PUSBController = ^TUSBController;
 TUSBController = object
  Public
   Index         : Word;
   BusNumber     : Byte;
   DevFuncNumber : Byte;
   BaseAddress   : LongInt;
   IRQNumber     : Byte;
  Public
   Procedure ResetController (Reset  : Boolean);
   Procedure EnableController(Enable : Boolean);
   Function  IsConfigured : Boolean;
   Procedure CommandStop;
   Procedure CommandRun;
   Procedure SetFrameNumber(Number : Word);
   Procedure SetFrameListAddress(FrameListAdr : LongInt);
  Public
   Function  GetPortBaseAddr(PortIndex : Byte) : Word;
   Function  GetPortState(PortIndex : Byte) : Word;
   Function  PortConnected(PortIndex : Byte): Boolean;
   Function  IsLowSpeedDevice(PortIndex : Byte): Boolean;
   Procedure EnablePort(PortIndex : Byte; Enable : Boolean);
 End;

{ ᨬ쭮 ᫮ ஫஢ }
Const
 MaxControllerIndex = 6;

{ ᮪ ஫஢ }
Type
 PUSBControllerList = ^TUSBControllerList;
 TUSBControllerList = object
  Private
   USBContrArr : Array [1..MaxControllerIndex] of PUSBController;
   USBContrCnt : Byte;
  Private
   Procedure DetectUSBDevice(ClassCode : Longint);
  Public
   Constructor Init;
   Destructor  Done; virtual;
  Public
   Procedure MakeList;
   Function  GetCount : Integer;
   Function  GetAt(Index : Integer) : PUSBController;
 End;

IMPLEMENTATION

{==================================================}
{                    TUSBController                }
{==================================================}
Procedure TUSBController.ResetController(Reset : Boolean);
Var Signal : Word;
Begin
 If Reset then Signal:= 4 else Signal:= 0;
 PortW[BaseAddress]:= Signal;
 Delay(10);
End;

Procedure TUSBController.EnableController(Enable : Boolean);
Var Signal : Word;
Begin
 If Enable then Signal:= 1 else Signal:= 0;
 PortW[BaseAddress]:= Signal;
End;

Function TUSBController.IsConfigured : Boolean;
Begin
 IsConfigured:= (BaseAddress <> -1);
End;

Function  TUSBController.GetPortBaseAddr(PortIndex : Byte) : Word;
Begin
 If PortIndex = 1
  Then GetPortBaseAddr:= BaseAddress + $10
  Else GetPortBaseAddr:= BaseAddress + $12;
End;

Function TUSBController.GetPortState(PortIndex : Byte) : Word;
Begin
 GetPortState:= PortW[GetPortBaseAddr(PortIndex)];
End;

Function TUSBController.PortConnected(PortIndex : Byte): Boolean;
Begin
 PortConnected:= (GetPortState(PortIndex) and $0001) <> 0;
End;

Function TUSBController.IsLowSpeedDevice(PortIndex : Byte): Boolean;
Begin
 IsLowSpeedDevice:= (GetPortState(PortIndex) and $0100) <> 0;
End;

Procedure TUSBController.EnablePort(PortIndex : Byte; Enable : Boolean);
Var PortAddr : Word;
Begin
 PortAddr:= GetPortBaseAddr(PortIndex);
 If Enable
  Then PortW[PortAddr]:= PortW[PortAddr] or  $04
  Else PortW[PortAddr]:= PortW[PortAddr] and $FB;
End;

procedure TUSBController.CommandRun;
begin
 PortW[BaseAddress]:= PortW[BaseAddress] or 1;
end;

procedure TUSBController.CommandStop;
begin
 PortW[BaseAddress]:= PortW[BaseAddress] and $FE;
end;

Procedure TUSBController.SetFrameNumber(Number : Word);
Begin
 PortW[BaseAddress+6]:= Number;
End;

Procedure TUSBController.SetFrameListAddress(FrameListAdr : LongInt);
Begin
 Asm
  mov dx, BaseAddress
  add dx, 08h
  db 66h; mov ax, word ptr FrameListAdr  { mov eax, ... }
  db 66h; out dx,ax                      { out dx,eax  }
 End;
End;

{==================================================}
{                 TUSBControllerList               }
{==================================================}
Constructor TUSBControllerList.Init;
Var Index : Integer;
Begin
 USBContrCnt:= 0;
 For Index:= 1 to MaxControllerIndex do
  USBContrArr[Index]:= nil;
End;

Destructor  TUSBControllerList.Done;
Var Index : Integer;
Begin
 For Index:= 1 to MaxControllerIndex do
  If USBContrArr[Index] <> nil then
   Dispose(USBContrArr[Index]);
End;

Procedure TUSBControllerList.MakeList;
Begin
 DetectUSBDevice($0C0300); {UHC  USB}
 {DetectUSBDevice($0C0310); {OHC  USB}
 {== EHCI    ᮢ!}
 {DetectUSBDevice($0C0320); {EHCI USB}
End;

Function  TUSBControllerList.GetCount : Integer;
Begin
 GetCount:= USBContrCnt;
End;

{  USB ஫஢ }
Procedure TUSBControllerList.DetectUSBDevice(ClassCode : Longint);
Var
 Index     : Word;
 BusNumber, DevFuncNumber, IRQNumber : Byte;
 BaseAddress : LongInt;
 Found : Byte;
 DeviceID, VendorID : Word;
Begin
 Index:= 0;
 While (True) do begin
  Found:= DetectPCIdeviceI(ClassCode, Index,
             BusNumber, DevFuncNumber);
  { ᫨  । ஫}
  {    ᯨ᮪ ஫஢}
  If Found = PCI_SUCCESS then begin
   {     ᯨ᮪ }
   Inc(USBContrCnt);
   USBContrArr[USBContrCnt]:= New(PUSBController);

   { 塞 ᢮⢠  }
   USBContrArr[USBContrCnt]^.Index        := Index;
   USBContrArr[USBContrCnt]^.BusNumber    := BusNumber;
   USBContrArr[USBContrCnt]^.DevFuncNumber:= DevFuncNumber;

   {20H - ᬥ饭  ᫮,}
   {࠭饣   ஫}
   If ReadPCIRegisterDWord($20,
       BusNumber, DevFuncNumber, BaseAddress) = PCI_SUCCESS then begin
    {㫨 5  }
    BaseAddress:= BaseAddress and $0FFE0;
    {࠭    ⠡}
    USBContrArr[USBContrCnt]^.BaseAddress:= BaseAddress;
   End else begin
    { 訡 ⥭   }
    USBContrArr[USBContrCnt]^.BaseAddress:= -1;
   End;

   {3CH - ᬥ饭 , ࠭饣 }
   { 뢠 IRQ}
   If ReadPCIRegisterByte($3C,
       BusNumber, DevFuncNumber, IRQNumber) = PCI_SUCCESS then begin
    USBContrArr[USBContrCnt]^.IRQNumber:= IRQNumber;
   End;

   If USBContrCnt > MaxControllerIndex then Exit;
  End else begin
   Break;
  End;
  Inc(Index);
 End;
End;

Function TUSBControllerList.GetAt(Index : Integer) : PUSBController;
Begin
 GetAt:= USBContrArr[Index];
End;

END.