unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, UsbTypes;

type
  TForm1 = class(TForm)
    LogListBox: TListBox;
    Panel1: TPanel;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    procedure Log(S : String);
  public
   procedure EnumerateHostDevices(hHost : THandle);
   procedure ShowRootHubInfo(RootHubName : String);
   procedure ShowRootHubDetail(hRoot : THandle);
   procedure ShowHubPortDetail(hRoot : THandle; iPort : Integer);
   procedure ShowDeviceDetail(hRoot : THandle; iPort : Integer);
   procedure DisplayDescriptorInfo(Data : Array of byte; var IsHid : Boolean);
   procedure DisplayREPORTDescriptorInfo(Data : Array of byte);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//    
procedure TForm1.Log(S : String);
begin
 LogListBox.Items.Add(S);
 LogListBox.ItemIndex:= LogListBox.Items.Count-1;
end;

//  
procedure TForm1.EnumerateHostDevices(hHost : THandle);
var Success : LongBool; DeviceName : TDeviceName;
    BytesReturned : Cardinal;
begin
 //     (IOCTL_GET_HCD_DRIVERKEY_NAME)
 Success:= DeviceIoControl(hHost, GetUSBCtlCode(10),
             @DeviceName, SizeOf(DeviceName),
             @DeviceName, sizeof(DeviceName),
             BytesReturned, nil
           );
 If not(Success) then begin
  Log('    ');
  Exit;
 End;

 Log(Format('   : %s',[WideCharToString(@DeviceName.Name)]));

 //     (IOCTL_USB_GET_ROOT_HUB_NAME)
 Success:= DeviceIoControl(hHost, GetUSBCtlCode(3),
             @DeviceName, SizeOf(DeviceName),
             @DeviceName, sizeof(DeviceName),
             BytesReturned, nil
           );
 If not(Success) then begin
  Log('    ');
  Exit;
 End;
 Log(Format('  . : %s',[WideCharToString(@DeviceName.Name)]));
 //     
 ShowRootHubInfo(WideCharToString(@DeviceName.Name));
end;

procedure TForm1.ShowRootHubInfo(RootHubName : String);
var sRoot : String; hRoot : THandle;
    SA : SECURITY_ATTRIBUTES;
begin
 //  SA
 SA.nLength:= SizeOf(SECURITY_ATTRIBUTES);
 SA.lpSecurityDescriptor:= nil;
 SA.bInheritHandle:= False;

 //       PnP- 
 sRoot:= Format('\\.\%s', [RootHubName]);
 //  
 hRoot:= CreateFile(PChar(@sRoot[1]),
          GENERIC_WRITE, FILE_SHARE_WRITE, @SA, OPEN_EXISTING, 0, 0
         );
 If (hRoot <> INVALID_HANDLE_VALUE) then begin
  //     
  ShowRootHubDetail(hRoot);
  CloseHandle(hRoot);
 End else begin
  Log('     ');
 End;
end;

//     
procedure TForm1.ShowRootHubDetail(hRoot : THandle);
var Success : LongBool; NodeInformation : TNodeInformation;
    BytesReturned : Cardinal; iPort : Byte;
begin
 //      (IOCTL_USB_GET_NODE_INFORMATION)
 Success:= DeviceIoControl(hRoot, GetUSBCtlCode(3),
            @NodeInformation, sizeof(NodeInformation),
            @NodeInformation, sizeof(NodeInformation),
            BytesReturned, nil
           );
 If not(Success) then begin
  Log('     ');
  Exit;
 End;
 //     
 Log(Format('   : %d', [NodeInformation.HubDescriptor.bNumberOfPorts]));
 For iPort:= 0 to NodeInformation.HubDescriptor.bNumberOfPorts-1 do begin
  //     
  ShowHubPortDetail(hRoot, iPort);
 End;
end;

//    
procedure TForm1.ShowHubPortDetail(hRoot : THandle; iPort : Integer);
var Success : LongBool; NodeConnInfo : TNodeConnectionInformation;
    BytesReturned : Cardinal; PortStatus : Byte;
begin
 NodeConnInfo.ConnectionIndex:= iPort+1; //   1!
 // IOCTL_USB_GET_NODE_CONNECTION_INFORMATION
 Success:= DeviceIoControl(hRoot, GetUSBCtlCode(4),
            @NodeConnInfo, sizeof(NodeConnInfo),
            @NodeConnInfo, sizeof(NodeConnInfo),
            BytesReturned, nil
           );
 If not(Success) then begin
  Log('       ');
  Exit;
 End;

 PortStatus:= NodeConnInfo.ConnectionStatus[1];
 If (PortStatus <> Byte(DeviceConnected)) then begin
  If (PortStatus = Byte(NoDeviceConnected)) then begin
   Log(Format('     %d   ',[iPort]));
  End else begin
   Log(Format('     %d = %d',[iPort, Byte(PortStatus)]));
  End; 
 End else begin
  If not(NodeConnInfo.DeviceIsHub) then begin
   Log(Format('     %d  ', [iPort]));
   ShowDeviceDetail(hRoot, iPort);
  End else begin
   Log(Format('     %d  ', [iPort]));
   //  IOCTL_USB_GET_NODE_CONNECTION_NAME
   //       
   //    
   // : ShowRootHubDetail(hRoot : THandle);
  End;
 End;
end;

//    
procedure TForm1.ShowDeviceDetail(hRoot : THandle; iPort : Integer);
Var Success : LongBool; Packet : TDescriptorRequest;
    BytesReturned : Cardinal; IsHID : Boolean;
begin
//    
 ZeroMemory(@Packet, SizeOf(Packet));
 Packet.ConnectionIndex       := iPort+1;
 Packet.SetupPacket.bmRequest := $80;
 Packet.SetupPacket.bRequest  := USB_REQUEST_GET_DESCRIPTOR;
 Packet.SetupPacket.wValue [2]:= USB_DEVICE_DESCRIPTOR_TYPE;
 Packet.SetupPacket.wLength[2]:= 1; //   2 

 // IOCTL_USB_GET_DESCRIPTOR_FROM_NODE_CONNECTION
 Success:= DeviceIoControl(hRoot, GetUSBCtlCode(5),
            @Packet, sizeof(Packet),
            @Packet, sizeof(Packet),
            BytesReturned, nil
           );

 If not(Success) then begin
  Log(Format('         %s', [ SysErrorMessage(GetLastError())]));
  Exit;
 End;

 IsHid:= False;

 //   
 DisplayDescriptorInfo(Packet.Data, IsHid);

//   
 ZeroMemory(@Packet, SizeOf(Packet));
 Packet.ConnectionIndex       := iPort+1;
 Packet.SetupPacket.bmRequest := $80;
 Packet.SetupPacket.bRequest  := USB_REQUEST_GET_DESCRIPTOR;
 Packet.SetupPacket.wValue [2]:= USB_CONFIGURATION_DESCRIPTOR_TYPE;
 Packet.SetupPacket.wLength[2]:= 1; //   2 

 // IOCTL_USB_GET_DESCRIPTOR_FROM_NODE_CONNECTION
 Success:= DeviceIoControl(hRoot, GetUSBCtlCode(5),
            @Packet, sizeof(Packet),
            @Packet, sizeof(Packet),
            BytesReturned, nil
           );

 If not(Success) then begin
  Log(Format('         %s', [ SysErrorMessage(GetLastError())]));
  Exit;
 End;

 //     
 DisplayDescriptorInfo(Packet.Data, IsHid);

 If IsHid then begin
    //   REPORT
     ZeroMemory(@Packet, SizeOf(Packet));
     Packet.ConnectionIndex       := iPort+1;
     Packet.SetupPacket.bmRequest := $80;
     Packet.SetupPacket.bRequest  := USB_REQUEST_GET_DESCRIPTOR;
     Packet.SetupPacket.wValue [2]:= USB_REPORT_DESCRIPTOR_TYPE;
     Packet.SetupPacket.wLength[2]:= 1; //   2 

     // IOCTL_USB_GET_DESCRIPTOR_FROM_NODE_CONNECTION
     Success:= DeviceIoControl(hRoot, GetUSBCtlCode(5),
                @Packet, sizeof(Packet),
                @Packet, sizeof(Packet),
                BytesReturned, nil
               );

     If not(Success) then begin
      Log(Format('      REPORT- %s', [ SysErrorMessage(GetLastError())]));
      Exit;
     End;

    //     
    DisplayREPORTDescriptorInfo(Packet.Data);
 End;
end;


procedure TForm1.DisplayDescriptorInfo(Data : Array of byte; var IsHid : Boolean);
var iData : Integer; lenDescr, typDescr : Byte; PData : Pointer;
begin
 //    
 //   -    
 //   -  
 iData:= 0;

 Repeat
  lenDescr:= Data[iData+0];
  typDescr:= Data[iData+1];
  PData   := @Data[iData+0];

  Case typDescr of
   $01: begin //  
     Log('       ');
     With TDeviceDescriptor(PData^) do begin
      Log(Format('      bcdUSB            =%s', [BCD2Str(bcdUSB)   ]));
      Log(Format('      bDeviceClass      =%d', [bDeviceClass      ]));
      Log(Format('      bDeviceSubClass   =%d', [bDeviceSubClass   ]));
      Log(Format('      bDeviceProtocol   =%d', [bDeviceProtocol   ]));
      Log(Format('      bMaxPacketSize0   =%d', [bMaxPacketSize0   ]));
      Log(Format('      idVendor          =%d', [idVendor          ]));
      Log(Format('      idProduct         =%d', [idProduct         ]));
      Log(Format('      bcdDevice         =%s', [BCD2Str(bcdDevice)]));
      Log(Format('      iManufacturer     =%d', [iManufacturer     ]));
      Log(Format('      iProduct          =%d', [iProduct          ]));
      Log(Format('      iSerialNumber     =%d', [iSerialNumber     ]));
      Log(Format('      bNumConfigurations=%d', [bNumConfigurations]));
     End;
    End;
   $02: begin //  
     Log('      ');
     With TUsbConfigurationDescriptor(PData^) do begin
      Log(Format('      iConfiguration=%d', [iConfiguration]));
      Log(Format('      bNumInterfaces=%d', [bNumInterfaces]));
      Log(Format('      bmAttributes  =%d', [bmAttributes  ]));
      Log(Format('      MaxPower()  =%d', [MaxPower*2    ]));
     End;
    End;
   $03: begin //  
     Log('      ');
    End;
    $04: begin //  
     Log('      ');
     With TUsbInterfaceDescriptor(PData^) do begin
      Log(Format('      bInterfaceClass=%d', [bInterfaceClass]));
      Log(Format('      bNumEndpoints  =%d', [bNumEndpoints  ]));
      IsHID:= (bInterfaceClass = 3);
     End;
    End;
    $05: begin //   
     Log('       ');
    End;
    $21: begin //  HID-
     Log('      HID-');
    End;
   End;

   iData:= iData + lenDescr;
  Until (lenDescr = 0) or (iData > High(Data));
end;

//  !
procedure TForm1.Button1Click(Sender: TObject);
var sHost : String; iHost : Integer; hHost : THandle;
    SA : SECURITY_ATTRIBUTES; EndOfSearch : Boolean;
begin
 //  SA
 SA.nLength:= SizeOf(SECURITY_ATTRIBUTES);
 SA.lpSecurityDescriptor:= nil;
 SA.bInheritHandle:= False;

 //    
 iHost:= 0;
 EndOfSearch:= False;
 Repeat
  //   
  sHost:= Format('\\.\HCD%d', [iHost]);
  Log(Format('  <%s>...',[sHost]));
  //   -
  hHost:= CreateFile(PChar(@sHost[1]),
           GENERIC_WRITE, FILE_SHARE_WRITE, @SA, OPEN_EXISTING, 0, 0
          );
  //   
  If (hHost <> INVALID_HANDLE_VALUE) then begin
   Log(Format('- %d  ',[iHost]));
   //  ,   
   EnumerateHostDevices(hHost);
   CloseHandle(hHost);
  End else begin
   Log(Format('- <%s>  ',[sHost]));
   EndOfSearch:= True;
  End;

  Inc(iHost);
 Until EndOfSearch;          
end;

procedure TForm1.DisplayREPORTDescriptorInfo(Data : Array of byte);
begin
 Log('      REPORT');
end;

end.
