#include <ntddk.h>
#include "testdrv.h"

//     
#define DEVICE_NAME_STRING L"testdriver"


//  -
VOID OnUnloadHandle(IN PDRIVER_OBJECT);
NTSTATUS IrpHandler(IN PDEVICE_OBJECT, IN PIRP);
NTSTATUS OnDeviceControlHandle(IN PDEVICE_OBJECT, IN PIRP);

NTSTATUS OnAddDeviceHandle(IN PDRIVER_OBJECT DriverObject, IN PDEVICE_OBJECT PhysicalDeviceObject);

/* ============================================================
   .   .
============================================================ */
void TestDriver(void)
{
 DbgPrint("TestDriver");
}

/* ============================================================
  .      
    . 
============================================================ */
NTSTATUS DriverEntry(
 IN PDRIVER_OBJECT DriverObject,
 IN PUNICODE_STRING RegistryPath
)
{
 PDEVICE_OBJECT deviceObject;
 NTSTATUS status;
 int i;

 WCHAR NameBuffer[] = L"\\Device\\" DEVICE_NAME_STRING;
 WCHAR DOSNameBuffer[] = L"\\DosDevices\\" DEVICE_NAME_STRING;
 UNICODE_STRING uniNameString, uniDOSString;

 DbgPrint("DriverEntry");

 //    
 RtlInitUnicodeString(&uniNameString, NameBuffer);
 RtlInitUnicodeString(&uniDOSString, DOSNameBuffer);

 //   
 status = IoCreateDevice(DriverObject, 0, &uniNameString,
		 FILE_DEVICE_UNKNOWN, 0, FALSE, &deviceObject);
 if(!NT_SUCCESS(status))
  return status;

 //    
 status = IoCreateSymbolicLink (&uniDOSString, &uniNameString);
 if (!NT_SUCCESS(status))
  return status;

 //       
 // . 
 for (i=0; i < IRP_MJ_MAXIMUM_FUNCTION; i++)
  DriverObject->MajorFunction[i]= IrpHandler;
 DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = OnDeviceControlHandle;
 DriverObject->DriverUnload = OnUnloadHandle;
 DriverObject->DriverExtension->AddDevice = OnAddDeviceHandle;

 return(STATUS_SUCCESS);
}

/* ============================================================
   CreateProcess()  
 .       
 DriverEntry,     
IRP_MJ_CREATE.
============================================================ */
NTSTATUS OnAddDeviceHandle(
 IN PDRIVER_OBJECT DriverObject,
 IN PDEVICE_OBJECT PhysicalDeviceObject 
)
{
 DbgPrint("OnAddDeviceHandle");
 return(STATUS_SUCCESS);
}

/* ============================================================
     , 
 IRP_MJ_DEVICE_CONTROL,  
============================================================ */
NTSTATUS IrpHandler(
 IN PDEVICE_OBJECT DeviceObject,
 IN PIRP Irp
)
{
 DbgPrint("IrpHandle");

 //  
 Irp->IoStatus.Information = 0;
 Irp->IoStatus.Status = STATUS_SUCCESS;
 IoCompleteRequest(Irp, IO_NO_INCREMENT);
 return STATUS_SUCCESS;
}

/* ============================================================
   .   
    DriverEntry,  
 DriverUnload.
============================================================ */
VOID OnUnloadHandle(IN PDRIVER_OBJECT DriverObject)
{
 //   
 WCHAR DOSNameBuffer[] = L"\\DosDevices\\" DEVICE_NAME_STRING;
 UNICODE_STRING uniDOSString;

 DbgPrint("OnUnloadHandle");

 //     
 RtlInitUnicodeString(&uniDOSString, DOSNameBuffer);
 //   
 IoDeleteSymbolicLink (&uniDOSString);
 //   
 IoDeleteDevice(DriverObject->DeviceObject);
}


/* ============================================================
   DeviceIoControl  
 .       
 DriverEntry,     
IRP_MJ_DEVICE_CONTROL.
============================================================ */
NTSTATUS OnDeviceControlHandle(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
    )

{
 NTSTATUS            ntStatus = STATUS_SUCCESS;   
 PIO_STACK_LOCATION  irpSp;

 //         
 ULONG  inBufLength;   
 //        
 ULONG  outBufLength;  
 //       
 PULONG ioBuffer;      

 DbgPrint("OnDeviceControlHandle");

 //    
 irpSp= IoGetCurrentIrpStackLocation(pIrp);

 //       
 inBufLength  = irpSp->Parameters.DeviceIoControl.InputBufferLength;
 outBufLength = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
 ioBuffer     = (PULONG) pIrp->AssociatedIrp.SystemBuffer;

 switch ( irpSp->Parameters.DeviceIoControl.IoControlCode )
 {
  //    IOCTL_IOPM_FUNCTION1
  case IOCTL_IOPM_FUNCTION1:
  {
   if (inBufLength > 0) {
    DbgPrint(">IOCTL_IOPM_FUNCTION1");
    ioBuffer[0]++;
   }
   break;
  }
  //    IOCTL_IOPM_FUNCTION2
  case IOCTL_IOPM_FUNCTION2:
  {
   if (inBufLength > 0) {
    DbgPrint(">IOCTL_IOPM_FUNCTION2");
    ioBuffer[0]--;
   }
   break;
  }
 }


 //   
 pIrp->IoStatus.Information = inBufLength; /*    */
 pIrp->IoStatus.Status = ntStatus;
 IoCompleteRequest( pIrp, IO_NO_INCREMENT );
 return ntStatus;
}
