unit DrvLoader;

interface

Uses Windows, SysUtils;

type
 TDriverLoader = class
  private
   hDevice : THandle;
  Public
   Constructor Create;  
   Destructor  Destroy; override;
  Public
   Function CreateService(SysPath : String) : Boolean;
   Function RemoveService : Boolean;

   Function StartService  : Boolean;
   Function StopService   : Boolean;
  public
   Function  Open : Boolean;
   Procedure Close;
  public
   Function Func1(Value : Cardinal) : Cardinal; 
   Function Func2(Value : Cardinal) : Cardinal; 
 End;

implementation

Uses WinSvc, Dialogs;

Constructor TDriverLoader.Create;
Begin
 Inherited Create;
 hDevice:= INVALID_HANDLE_VALUE;
End;

Destructor  TDriverLoader.Destroy;
Begin
 Close;
 Inherited Destroy;
End;

Const
 DriverName : PChar= 'testdriver'#0;
 FileDriver : String = 'testdrv.sys'#0;
 
Function TDriverLoader.CreateService(SysPath : String) : Boolean;
var lpServiceArgVectors : PChar;
    hSCMan, hService: SC_HANDLE;
    DriverPath : String;
Begin
 Result:= False;
 
 {==   ==}
 {    }
 {HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\testdriver}
 {  :}
 { DisplayName = _ ( testdriver)}
 { ImagePath   =    sys.      }
 {                 \??\E:\Test\testdrv.sys}
 hSCMan:= WinSvc.OpenSCManager(
           Nil,         { local }
           Nil,         { SERVICES_ACTIVE_DATABASE }
           SC_MANAGER_ALL_ACCESS
          );
 If hSCMan = 0 then Exit;        

 {      }
 DriverPath:= SysPath + FileDriver;

 {  ( )}
 hService:= WinSvc.CreateService(hSCMan, Drivername, DriverName,
      SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER,
      SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,
      PChar(@DriverPath[1]),
      nil,nil,nil,nil,nil);
 {      }     
 If hService = 0 then begin
  MessageDlg(SysErrorMessage(GetLastError), mtError, [mbOK], 0);
  CloseServiceHandle(hSCMan);
  Exit;
 End;

 { }
 WinSvc.CloseServiceHandle(hService);
 WinSvc.CloseServiceHandle(hSCMan);
 Result:= True;
End;

Function TDriverLoader.RemoveService : Boolean;
var serviceStatus    : TServiceStatus;
    hSCMan, hService : SC_HANDLE;
Begin
 Result:= False;
 {==    }
 hSCMan:= WinSvc.OpenSCManager(Nil,Nil,SC_MANAGER_ALL_ACCESS);
 If hSCMan = 0 then Exit;

 hService:= WinSvc.OpenService(hSCMan, DriverName, SERVICE_ALL_ACCESS);
 {  }
 If hService=0 then begin
  CloseServiceHandle(hSCMan);
  Exit;
 End;

 WinSvc.DeleteService(hService);
 WinSvc.CloseServiceHandle(hService);
 WinSvc.CloseServiceHandle(hSCMan);
 Result:= True;
End;

Function TDriverLoader.StartService  : Boolean;
var lpServiceArgVectors : PChar;
    hSCMan, hService : SC_HANDLE;
    DriverPath : String;
Begin
 Result:= False;
 
 {==   }
 hSCMan  := WinSvc.OpenSCManager(Nil, Nil, SC_MANAGER_CONNECT);
 If hSCMan = 0 then Exit;
 
 hService:= WinSvc.OpenService(hSCMan, DriverName, SERVICE_START);
 If hService = 0 then begin
  CloseServiceHandle(hSCMan);
  Exit;
 End; 

 lpServiceArgVectors:=nil;
 WinSvc.StartService(hService, 0, lpServiceArgVectors);
 WinSvc.CloseServiceHandle(hService);
 WinSvc.CloseServiceHandle(hSCMan);
 Result:= True;
End;

Function TDriverLoader.StopService   : Boolean;
var serviceStatus    : TServiceStatus;
    hSCMan, hService : SC_HANDLE;
Begin
 Result:= False;
 {==  }
 hSCMan:= WinSvc.OpenSCManager(Nil, Nil, SC_MANAGER_CONNECT);
 If hSCMan = 0 then Exit;

 hService:= WinSvc.OpenService(hSCMan, DriverName, SERVICE_STOP);
 If hService = 0 then begin
  WinSvc.CloseServiceHandle(hSCMan);
  Exit;
 End;

 WinSvc.ControlService(hService, SERVICE_CONTROL_STOP, serviceStatus);
 WinSvc.CloseServiceHandle(hService);
 WinSvc.CloseServiceHandle(hSCMan);
 Result:= True;
End;

Function TDriverLoader.Open : Boolean;
Begin
 hDevice:= CreateFile('\\.\testdriver',
             GENERIC_READ or GENERIC_WRITE,
             0,nil,
             OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL,
             0
           );
 Result:= hDevice <> INVALID_HANDLE_VALUE;
End;

Procedure TDriverLoader.Close;
Begin
 If hDevice <> INVALID_HANDLE_VALUE then
  CloseHandle(hDevice);
 hDevice:= INVALID_HANDLE_VALUE;
End;

Const
 TESTDRV_TYPE    =  40000;
 METHOD_BUFFERED = 0;
 FILE_ANY_ACCESS = $0000;

function Get_Ctl_Code(Nr: Integer): Integer;
begin
  Result:=
     (TESTDRV_TYPE    shl 16) or
     (FILE_ANY_ACCESS shl 14) or
     (Nr              shl  2) or
     METHOD_BUFFERED;
end;

Function TDriverLoader.Func1(Value : Cardinal) : Cardinal;
Var Res : Cardinal;
Begin
 If hDevice = INVALID_HANDLE_VALUE then Exit;
 DeviceIoControl(hDevice, Get_Ctl_Code($800),
    @Value, SizeOf(Cardinal), @Result, SizeOf(Cardinal), Res, nil);
End;

Function TDriverLoader.Func2(Value : Cardinal) : Cardinal;
Var Res : Cardinal;
Begin
 If hDevice = INVALID_HANDLE_VALUE then Exit;
 DeviceIoControl(hDevice, Get_Ctl_Code($801),
    @Value, SizeOf(Cardinal), @Result, SizeOf(Cardinal), Res, nil);
End;

end.
